// RemotePhotoClient.java

import ch.aplu.jgamegrid.*;
import java.io.*;
import ch.aplu.bluetooth.*;
import java.awt.image.BufferedImage;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import javax.bluetooth.RemoteDevice;
import javax.swing.UIManager;
import com.l2fprod.common.swing.plaf.LookAndFeelAddons;
import java.awt.event.KeyEvent;
import javax.swing.JOptionPane;

public class RemotePhotoClient extends GameGrid
  implements GGExitListener, GGButtonListener, GGKeyListener
{
  private final String serviceName = "PhotoTransfer";
  private final int exitCommand = -1;  // Send to server to signal that we exit 
  private byte[] eof =  // End of file
  {
    (byte)0xFF, (byte)0xD9
  };
  private byte[] eot =  // End fo transmission
  {
    (byte)0xFF, (byte)0xFE
  };
  private BluetoothClient bc;
  private InputStream is;
  private DataOutputStream dos;
  private final String userHome = System.getProperty("user.home");
  private final String fs = System.getProperty("file.separator");
  private String propPath = userHome + fs + "RemotePhotoClient.properties";
  private final File photoDir = new File(userHome + fs + "GGPhotoCapture");
  private File photoFile;
  private int size;
  private boolean isConnected = false;
  private GGButton[] buttons =
  {
    new GGButton("sprites/100.gif", true),
    new GGButton("sprites/80.gif", true),
    new GGButton("sprites/60.gif", true),
    new GGButton("sprites/40.gif", true),
    new GGButton("sprites/20.gif", true)
  };
  private Actor image = null;
  private Actor waitActor = new Actor("sprites/waitimage.png");
  private Actor splashActor = new Actor("sprites/splash.png");
  private Location showLocation = new Location(300, 200);
  private Location hideLocation = new Location(-300, 200);
  private File propFile = new File(propPath);
  private Properties prop = new Properties();
  private boolean isKeyEnabled = false;
  private boolean isKeyReleased = true;

  public RemotePhotoClient()
  {
    super(600, 600, 1, false);
    addStatusBar(30);
    addActor(waitActor, hideLocation);
    addActor(splashActor, showLocation);
    int x = 50;
    for (GGButton btn : buttons)
    {
      btn.addButtonListener(this);
      btn.setEnabled(false);
      addActor(btn, new Location(x, 500));
      x += 60;
    }
    addExitListener(this);
    addKeyListener(this);

    Properties prop = loadProperties();
    String btName = "";
    if (prop != null)
    {
      btName = prop.getProperty("BluetoothName");
      if (btName == null)
        btName = "";
      else
        btName = btName.trim();
    }
    show();

    boolean rc = GGPath.createDirectoryTree(photoDir);
    if (!rc)
    {
      setStatusText("Can't create photo directory " + photoDir);
      System.out.println("Can't create photo directory " + photoDir);
      return;
    }

    setStatusText("Select server's Bluetooth name");
    String prompt = "Enter Server's Bluetooth Name";
    String serverName;
    do
    {
      serverName = JOptionPane.showInputDialog(null, prompt, btName);
      if (serverName == null)
        System.exit(0);
    }
    while (serverName.trim().length() == 0);
    serverName = serverName.trim();
    setProperty("BluetoothName", serverName);

    System.out.println("Trying to find paired device '" + serverName + "'");
    setStatusText("Trying to find paired device '" + serverName + "'");

    // Try to get device from paired devices database
    RemoteDevice rd = BluetoothFinder.searchPreknownDevice(serverName);
    if (rd == null)
    {
      System.out.println("Sorry. Device '" + serverName + "' not paired.");
      System.out.println("Please perform Bluetooth device pairing!");
      setStatusText("Device '" + serverName + "' not paired. Perform pairing");
      return;
    }
    System.out.println("Paired device found. Searching service now...");
    setStatusText("Paired device found. Searching service now...");
    bc = new BluetoothClient(rd, serviceName);
    bc.setVerbose(true);
    System.out.println("Connecting  to '" + serverName + "' ...");
    setStatusText("Connecting  to '" + serverName + "' ...");
    if (!bc.connect())
    {
      System.out.println("Connection failed");
      setStatusText("Connection failed");
      return;
    }
    System.out.println("Connection established");

    delay(2000);
    setStatusText("Connection established");
    delay(3000);
    is = bc.getInputStream();
    dos = new DataOutputStream(bc.getOutputStream());
    isConnected = true;
    while (true)
    {
      rc = receiveFile();
      if (!rc)
      {
        System.out.println("Server shut down");
        isConnected = false;
        setCommandEnabled(false);
        return;
      }
      if (size == 0)
        return;
      System.out.println("File transfer finished. Size = " + size);
      System.out.println("Display it now...");
      setStatusText("Transfer finished. Size = " + size + " bytes. Displaying it...");
      // Display photo
      BufferedImage bi = GGBitmap.getImage(photoFile.getPath());
      int height = bi.getHeight();
      int width = bi.getWidth();
      double zoomFactor = 600.0 / Math.max(height, width);
      bi = GGBitmap.getScaledImage(bi, zoomFactor, 0);
      image = new Actor(bi);
      addActorNoRefresh(image, new Location(300, 200));
      setPaintOrder(GGButton.class);
      waitActor.setLocation(hideLocation);
      refresh();
    }
  }

  public void buttonClicked(GGButton button)
  {
    if (button == buttons[0])
      sendCommand(100);
    if (button == buttons[1])
      sendCommand(80);
    if (button == buttons[2])
      sendCommand(60);
    if (button == buttons[3])
      sendCommand(40);
    if (button == buttons[4])
      sendCommand(20);
    setCommandEnabled(false);
  }

  public void buttonPressed(GGButton button)
  {
  }

  public void buttonReleased(GGButton button)
  {
  }

  private void sendCommand(int command)
  {
    setStatusText("Sending command to take snapshot with " + command + "%");
    waitActor.setLocation(showLocation);
    splashActor.setLocation(hideLocation);
    if (image != null)
      removeActor(image);
    refresh();
    try
    {
      dos.writeInt(command);
      dos.flush();
    }
    catch (IOException ex)
    {
      setStatusText("Exception while sending command");
    }
  }

  public boolean notifyExit()
  {
    System.out.println("notifyExit()");
    if (!isConnected)
      return true;
    try
    {
      System.out.println("Send command " + exitCommand);
      dos.writeInt(exitCommand);
      dos.flush();
    }
    catch (IOException ex)
    {
    }
    System.out.println("Disconnecting");
    bc.disconnect();
    delay(1000);
    return true;
  }

  private boolean receiveFile()
  {
    byte lastByte = 0;
    System.out.println("Waiting for command");
    setStatusText("Press button (or key 2,4,6,8,0) to take a snapshot (in percents of camera resolution).");
    setCommandEnabled(true);

    OutputStream out = null;
    size = 0;
    try
    {
      byte[] buf = new byte[8192];
      int len;
      boolean isEof = false;
      while (!isEof)
      {
        len = is.read(buf);  // Blocking read

        if (out == null)
        {
          SimpleDateFormat df = new SimpleDateFormat("yyyyMMddkkmmss");
          String date = df.format(new Date());
          String filename = "Capture_" + date + ".jpg";
          photoFile = new File(photoDir, filename);
          System.out.println("photoFile =  " + photoFile.getPath());
          out = new FileOutputStream(photoFile.getPath());
        }

        if (len == -1)
        {
          System.out.println("len = -1");
          isEof = true;
          continue;
        }

        out.write(buf, 0, len);
        size += len;
        System.out.println("Returned. Len: " + len + " size = " + size);
        if (size % 16384 == 0)
          setStatusText("# bytes transferred: " + size);

        if (len > 1)
        {
          if (buf[len - 2] == eof[0] && buf[len - 1] == eof[1])
          {
            System.out.println("EOF detected");
            isEof = true;
            setStatusText("EOF detected");
          }

          if (buf[len - 2] == eot[0] && buf[len - 1] == eot[1])
          {
            System.out.println("EOT found");
            throw new IOException("Server is closing");
          }
        }
        else
        {
          if (lastByte == eof[0] && buf[0] == eof[1])
          {
            System.out.println("EOF found");
            isEof = true;
            setStatusText("EOF detected");
          }
          if (lastByte == eot[0] && buf[0] == eot[1])
          {
            System.out.println("EOT found");
            throw new IOException("Server is closing");
          }
        }

        lastByte = buf[len - 1];
      }
      System.out.println("# of bytes transferred: " + size);
      setStatusText("# of bytes transferred: " + size);
    }
    catch (IOException ex)
    {
      System.out.println("Exception in receiveFile() " + ex.getMessage());
      setStatusText("Connection lost");
      return false;
    }
    finally
    {
      try
      {
        out.close();
      }
      catch (Exception ex)
      {
      }
      System.out.println("File closed");
    }
    return true;
  }

  private void setCommandEnabled(boolean enable)
  {
    for (GGButton button : buttons)
    {
      button.setEnabled(enable);
    }
    isKeyEnabled = enable;
  }

  private void setButtonsReleased()
  {
    for (GGButton button : buttons)
    {
      button.show(0);
    }
  }

  private Properties loadProperties()
  {
    // Return null, if error
    if (!propFile.exists())
    {
      try
      {
        propFile.createNewFile();
      }
      catch (IOException ex)
      {
        return null;
      }
    }
    FileInputStream fis = null;
    try
    {
      fis = new FileInputStream(propFile);
      prop.load(fis);
    }
    catch (IOException ex)
    {
      return null;
    }
    finally
    {
      try
      {
        fis.close();
      }
      catch (Exception ex)
      {
      }
    }
    return prop;
  }

  private void setProperty(String key, String value)
  {
    if (prop == null)
      return;
    prop.setProperty(key, value);
    try
    {
      FileOutputStream fos = new FileOutputStream(propFile);
      prop.store(fos, null);
      fos.close();
    }
    catch (IOException ex)
    {
      System.out.println("Can't set property " + key);
    }
  }

  public boolean keyPressed(KeyEvent evt)
  {
    if (!(isKeyEnabled && isKeyReleased))
      return true;
    isKeyReleased = false;
    isKeyEnabled = false;
    setCommandEnabled(false);
    int code = evt.getKeyCode();
    switch (code)
    {
      case KeyEvent.VK_2:
        sendCommand(20);
        buttons[4].show(1);
        break;
      case KeyEvent.VK_4:
        sendCommand(40);
        buttons[3].show(1);
        break;
      case KeyEvent.VK_6:
        sendCommand(60);
        buttons[2].show(1);
        break;
      case KeyEvent.VK_8:
        sendCommand(80);
        buttons[1].show(1);
        break;
      case KeyEvent.VK_0:
        sendCommand(100);
        buttons[0].show(1);
        break;
    }
    refresh();
    return true;  // Consumed
  }

  public boolean keyReleased(KeyEvent evt)
  {
    isKeyReleased = true;
    setButtonsReleased();
    refresh();
    return true;  // Consumed
  }

  public static void main(String[] args)
  {
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      LookAndFeelAddons.setAddon(LookAndFeelAddons.getBestMatchAddonClassName());
    }
    catch (Exception e)
    {
    }
    new RemotePhotoClient();
  }

}
