// BtPeerEx1.java

package ch.aplu.btpeerex1;

import android.bluetooth.*;
import android.graphics.Color;
import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;

public class BtPeerEx1 extends GameGrid
  implements GGTouchListener, BluetoothPeerListener
{
  private enum State
  {
    READY, GAME_OVER, CONNECTION_LOST
  }

  private interface Command
  {
    int all_sunk = -1;
    int sunk = -2;
    int disconnect = -3;
  }

  private final String serviceName = "BtPeerEx1";
  private BluetoothPeer bp;
  private final int DKGREEN = Color.rgb(0, 80, 0);
  private final int ORANGE = Color.rgb(255, 80, 0);
  private GGStatusBar status;
  private volatile State state = State.READY;
  private final String msgMyShoot = "Tap a cell to fire";
  private final String msgEnemyShoot = "Please wait enemy bomb";
  private boolean isWinner;
  private Location touchLocation;

  public BtPeerEx1()
  {
    super(10, 1, cellZoom(40), RED, null, false);
    setScreenOrientation(LANDSCAPE);
    status = addStatusBar(30);
  }

  public void main()
  {
    status.setRefreshEnabled(true);
    setSimulationPeriod(50);
//    setTouchEnabled(false);
    addTouchListener(this, GGTouch.press);
    deploy();
    connect();
    while (true)
    {
      if (state == State.GAME_OVER)
      {
        deploy();
        if (isWinner)
        {
          status.setText("You won! " + msgEnemyShoot, BLUE);
          setTouchEnabled(false);
        }
        else
        {
          status.setText("You lost! " + msgMyShoot, BLUE);
          setTouchEnabled(true);
        }
        state = State.READY;
      }
      delay(500);
    }
  }

  private void deploy()
  {
    removeAllActors();
    for (int i = 0; i < 3; i++)
      addActor(new Ship(), getRandomEmptyLocation());
  }

  public boolean touchEvent(GGTouch touch)
  {
    touchLocation = toLocation(touch.getX(), touch.getY());
    if (!isInGrid(touchLocation))
      return false;
    setTouchEnabled(false);
    addActor(new Actor("marker"), touchLocation);
    int[] data = new int[1];
    data[0] = touchLocation.x; // cell coordinate
    bp.sendDataBlock(data);
    status.setText(msgEnemyShoot, RED);
    return false;
  }

  private void connect()
  {
    status.setText("BtBattleShip", DKGRAY);
    String partnerName = askName();
    status.setText("Searching for partner name '" + partnerName + "' ...", DKGRAY);
    BluetoothDevice partnerDevice = searchDevice(partnerName);
    if (partnerDevice != null)
    {
      status.setText("Trying to connect to '" + partnerName + "' ...");
      bp = new BluetoothPeer(this, partnerDevice, serviceName, this, true);
      if (bp.isConnected())
      {
        status.setText("Connection established. " + msgMyShoot, DKGREEN);
        setTouchEnabled(true);
      }
      else
        status.setText("'" + partnerName + "' not found. Waiting as server '"
          + BluetoothDiscovery.getBluetoothAdapter(this).getName() + "' ...",
          ORANGE);
    }
  }

  public void notifyConnection(BluetoothDevice device, boolean connected)
  {
    if (connected)
    {
      status.setText("Connection established. Wait for enemy's shoot", RED);
    }
    else
    {
      status.setText("Connection lost. Press [HOME] and restart the app.", RED);
      state = State.CONNECTION_LOST;
      setTouchEnabled(false);
    }
  }

  // Signal to server that we quit
  public void onPause()
  {
    L.i("Custom onPause");

    // Because closing the Bluetooth socket is not
    // detected by every partner, we send an 'disconnect' command
    if (bp != null && bp.isConnected())
    {
      sendCommand(Command.disconnect);
      bp.releaseConnection();
      state = State.CONNECTION_LOST;
    }
    super.onPause();
  }

  public void receiveDataBlock(int[] data)
  {
    int x = data[0];
    // Command dispatcher
    switch (data[0])
    {
      case Command.disconnect:
        notifyConnection(null, false);
        return;
      case Command.all_sunk:
        state = State.GAME_OVER;
        isWinner = true;
        return;
      case Command.sunk:
        addActor(new Actor("hit"), touchLocation);
        return;
        
      default: // Got 'location' command
        Actor actor = getOneActorAt(new Location(x, 0), Ship.class);
        {
          if (actor != null)
          {
            actor.removeSelf();
            sendCommand(Command.sunk);
          }
        }
        break;
    }

    if (getNumberOfActors(Ship.class) == 0)
    {
      sendCommand(Command.all_sunk);
      state = State.GAME_OVER;
      isWinner = false;
      return;
    }
    
    status.setText(msgMyShoot, DKGREEN);
    setTouchEnabled(true);
  }

  private void sendCommand(int command)
  {
    int[] data = new int[1];
    data[0] = command;
    bp.sendDataBlock(data);
  }

  private String askName()
  {
    GGPreferences prefs = new GGPreferences(this);
    String oldName = prefs.retrieveString("BluetoothName");
    String newName = null;
    while (newName == null)
    {
      newName = GGInputDialog.show(this, "Search Device",
        "Enter Bluetooth Name",
        oldName == null ? "" : oldName);
    }
    prefs.storeString("BluetoothName", newName);
    return newName;
  }

  // Searches for device in set of paired devices. Returns null, if search fails.
  private BluetoothDevice searchDevice(String btName)
  {
    BluetoothAdapter adapter =
      BluetoothDiscovery.getBluetoothAdapter(this);
    if (adapter == null)
    {
      status.setText("Bluetooth not supported", RED);  // e.g. in emulator
      return null;
    }
    if (!adapter.isEnabled())
    {
      requestBluetoothEnable();
      if (!adapter.isEnabled())
      {
        status.setText("Can' enable Bluetooth", RED);
        return null;
      }
    }
    status.setText("Searching for paired device '" + btName + "'...");
    BluetoothDiscovery bd = new BluetoothDiscovery(this);
    BluetoothDevice device = bd.getPairedDevice(btName);
    if (device != null)
      status.setText("Paired device '" + btName + "' found.");
    else
      status.setText("Please pair with device '" + btName + "'");
    return device;
  }
  
  
  // ------------------ Class Ship ------------------
  class Ship extends Actor
  {
    public Ship()
    {
      super("ship");
    }
  }

}
