// RemotePhoto.java

package ch.aplu.remotephoto;

import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import android.bluetooth.*;
import android.graphics.Bitmap;
import android.hardware.Camera;
import java.io.*;
import android.os.Environment;
import java.text.SimpleDateFormat;
import java.util.Date;

public class RemotePhoto extends GameGrid
  implements BluetoothServerListener
{
  private class ReceiverThread extends Thread
  {
    public void run()
    {
      L.i("Receiver thread started");
      while (clientCommand != exitCommand)
      {
        try
        {
          L.i("Calling blocking readInt()");
          clientCommand = dis.readInt();  // Blocking, wait for clientCommand
        }
        catch (IOException ex)
        {
          L.i("IOExeption in blocking readInt()");
          clientCommand = exitCommand;
        }
        status.setText("Got command: " + clientCommand);
        L.i("Got command: " + clientCommand);
        if (clientCommand == exitCommand)
          isReadyToServe = false;
      }
      L.i("Receiver thread terminated");
    }

  }

  private class TransmitterThread extends Thread
  {
    public void run()
    {
      L.i("Transmitter thread started");
      status.setText("Sending file now...");
      boolean rc = sendFile(photoFile, dos);
      if (!rc)
      {
        status.setText("Error while sending file");
        L.i("Error while sending file.");
      }
      else
        status.setText("Fle transmission successfully finished. Size = " + size);

      L.i("Transmitter thread terminated");
    }

  }

  //
  private ReceiverThread inputListener = null;
  private final int idleCommand = 0;
  private final int exitCommand = -1;
  private int clientCommand = idleCommand;
  private final String serviceName = "PhotoTransfer";
  private File photoFile;
  private BluetoothServer server;
  private BluetoothAdapter adapter;
  private int size;
  private DataInputStream dis;
  private DataOutputStream dos;
  private TransmitterThread t;
  private GGStatusBar status;
  private boolean isReadyToServe;
  private GGCamera camera;

  public RemotePhoto()
  {
    super(WHITE, false, true, null);
    setScreenOrientation(LANDSCAPE);
    status = addStatusBar(20);
  }

  public void main()
  {
    status.setRefreshEnabled(true);
    camera = GGCamera.open(this);

    adapter = BluetoothDiscovery.getBluetoothAdapter(this);
    status.setText("Creating new BluetoothServer...");
    server = new BluetoothServer(adapter, serviceName, this);
    status.setText("Waiting for the Bluetooth client to connect...");
  }

  public boolean notifyClientConnection(BluetoothDevice device,
    InputStream is, OutputStream os)
  {
    L.i("notifyClientConnection() starting");
    dos = new DataOutputStream(os);
    dis = new DataInputStream(is);
    clientCommand = idleCommand;
    inputListener = new ReceiverThread();
    inputListener.start();
    isReadyToServe = true;
    while (isReadyToServe)
    {
      status.setText("Client '" + device.getName()
        + "' connected. Waiting for command...");
      while (clientCommand == idleCommand)
        delay(100);

      if (clientCommand == exitCommand)
      {
        try
        {
          dis.close(); // Stops ReceiverThread
        }
        catch (IOException ex)
        {
        }
        isReadyToServe = false;
        continue;
      }

      status.setText("Taking snapshot...");
      L.i("Calling takeSnapshot()");
      Bitmap bm = camera.takeSnapshot();
      double zoomFactor = clientCommand / 100.0;
      Bitmap scaledBm = GGBitmap.getZoomedImage(bm, zoomFactor);
      status.setText("Writing JPEG to to sdcard...");
      photoFile = getPhotoFile();
      boolean success = GGPath.writeImageFile(scaledBm, photoFile, "JPG", true);
      if (!success || photoFile.length() == 0)
      {
        status.setText("Failed to write image to sdcard");
        break;
      }

      status.setText("Capture photo size: " + photoFile.length());
      Camera.Size size = camera.getCamera().getParameters().getPictureSize();
      L.i("picture width = " + size.width + " , picture height = " + size.height);
      zoomFactor = Math.min(1.0, (double)getNbVertCells() / size.height);
      scaledBm = GGBitmap.getZoomedImage(bm, zoomFactor);
      Actor actor = new Actor(scaledBm);
      addActor(actor, new Location(getNbHorzCells() / 2, getNbVertCells() / 2));

      t = new TransmitterThread();
      t.start();
      while (t.isAlive())
        delay(1000);
      clientCommand = idleCommand;
    }
    status.setText("Lost client connection. Waiting for next client...");
    return true;  // Wait for next client connection
  }

  // Will by called by system
  public void onPause()
  {
    L.i("Custom onPause");
    if (server != null)
    {
      sendShutDownInfo();
    }
    camera.release();
    super.onPause();
  }

  private void sendShutDownInfo()
  {
    if (dos == null)
      return;
    try
    {
      // Comment tag for JPEG
      // Here used for End of Transmission clientCommand (EOT)
      dos.write((byte)0xFF);
      dos.write((byte)0xFE);
      dos.flush();
    }
    catch (IOException ex)
    {
    }
  }

  private boolean sendFile(File srcFile, DataOutputStream dos)
  {
    InputStream in = null;
    size = 0;
    try
    {
      in = new FileInputStream(srcFile);
    }
    catch (IOException ex)
    {
      L.i("Error while opening photo file: " + srcFile.getPath());
      sendShutDownInfo();
      return false;
    }

    try
    {
      byte[] buf = new byte[256];
      int len;
      while ((len = in.read(buf)) > 0)
      {
        if (clientCommand == exitCommand)
          throw new IOException("File transmission interrupted by client");
        dos.write(buf, 0, len);
        delay(10);  // Must delay to give up the thread
        size += len;
        if (size % 16384 == 0)
        {
          status.setText("# bytes transferred: " + size);
          L.i("# bytes transferred: " + size);
        }
      }
      dos.flush();
      status.setText("Transfer finished. Size = " + size);
      L.i("Transfer finished. Size = " + size);
    }
    catch (IOException ex)
    {
      status.setText("Transfer interrupted");
      L.i("Exception in sendFile: " + ex);
      return false;
    }
    return true;
  }
  
  private File getPhotoFile()
  {  
    File photoDir = 
      Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES);
    SimpleDateFormat df = new SimpleDateFormat("yyyyMMddkkmmss");
    String date = df.format(new Date());
    String filename = "Capture_" + date + ".jpg";
    return new File(photoDir, filename);
  }
}
